﻿using UnityEngine;
using UnityEngine.SceneManagement;
using System.Collections;

public enum Orientation
{
    UP,
    RIGHT,
    DOWN,
    LEFT
}

public class GameSceneManager : MonoBehaviour
{
    #region StaticRegion
    public static GameSceneManager Instance { get; private set; }
    private static bool UpdateMonitoredTiles;
    public static void RequestUpdateMonitoredTiles()
    {
        UpdateMonitoredTiles = true;
    }
    public static int   Money { get; private set; }
    [SerializeField]
    private int startMoney;
    #endregion

    public SpecialBuildingsManager  specialBuildingsManager;
    public BuildingManager          buildingManager;
    public PatrolManager            patrolManager;
    public GridManager              gridManager;
    public CrimeManager             crimeManager;
    public EconomyManager           economyManager;

    public enum GameStateType
    {
        NORMAL,
        BUYING_POLICE_CAMERA,
        BUYING_POLICE_MAN,
        BUYING_POLICE_STATION,
        BUYING_POLICE_CAR,
        BUYING_SCHOOL,
        EDITING_POLICE_CAMERA,
        EDITING_POLICE_MAN,
        EDITING_POLICE_STATION,
        EDITING_POLICE_CAR,
        EDITING_SCHOOL,
        END_GAME
    }
    public static GameStateType GameState;

    public PoliceUnit selectedUnit;
    public PoliceStation selectedStation;
    public PoliceCamera selectedCamera;

    public float gameTime = 0f;

    public GameObject   endScreenContanier;
    public GameObject   winContanier;
    public GameObject   lossContanier;

    public int          lossThreshold;

    void Start ()
    {
        Instance = this;
        Time.timeScale = 1f;
        GameState = GameStateType.NORMAL;
        Money = startMoney;
        buildingManager.OnBuildingClicked += OnBuildingClicked;
        gridManager.OnStreetTileClicked += OnStreetTileClicked;
        gridManager.OnBuildingTileClicked += OnBuildingTileClicked;
        gridManager.OnCrimeStarted += OnCrimeStarted;
    }

    private void OnCrimeStarted(GridTile p_tile)
    {
        crimeManager.SpawnCrime(p_tile);
        crimeManager.CheckSeenCrimes();
        crimeManager.CheckStoppedCrimes();
    }

    private void Update()
    {
        if (GameState == GameStateType.END_GAME)
            Time.timeScale = 0f;
        //else if (Input.GetKey(KeyCode.Space))
        //   Time.timeScale = 25f;
        else
            Time.timeScale = 1f;
        if (UpdateMonitoredTiles)
        {
            UpdateMonitoredTiles = false;
            gridManager.UpdateMonitoredTiles(patrolManager.policeUnits, patrolManager.policeCameras,
                specialBuildingsManager.policeStations);
            crimeManager.CheckSeenCrimes();
            crimeManager.CheckStoppedCrimes();
        }

        if (GameState == GameStateType.END_GAME)
            return;
        gameTime += Time.deltaTime;
        UIManager.Instance.UpdateTimeLabel(Mathf.FloorToInt(gameTime / 8f) + 1);
        if (gameTime > 240f)
            ShowEndGameScreen(true);
        if (crimeManager.seenCrimes + crimeManager.notseenCrimes >= lossThreshold)
            ShowEndGameScreen(false);

    }
    
    private void OnBuildingTileClicked(GridTile p_tile)
    {
        if (GameState == GameStateType.BUYING_POLICE_STATION && HaveEnoughMoney(2))
        {
            specialBuildingsManager.SpawnPoliceStation(p_tile);
            Money -= economyManager.prices[2];
            UIManager.Instance.UpdateMoneyLabel();
            ReturnToNormalState();
        }
    }
   
    private void OnStreetTileClicked(GridTile p_tile)
    {
        if (GameState == GameStateType.BUYING_POLICE_CAMERA && HaveEnoughMoney(0))
        {
            patrolManager.SpawnCamera(p_tile);
            Money -= economyManager.prices[0];
            UIManager.Instance.UpdateMoneyLabel();
            ReturnToNormalState();
        }
    }

    private void OnBuildingClicked(Building p_building)
    {
        if (GameState == GameStateType.BUYING_POLICE_MAN && HaveEnoughMoney(1))
        {
            patrolManager.SpawnPoliceMen(p_building);
            Money -= economyManager.prices[1];
            UIManager.Instance.UpdateMoneyLabel();
            ReturnToNormalState();
        }
        else if (GameState == GameStateType.BUYING_POLICE_CAR && HaveEnoughMoney(3))
        {
            patrolManager.SpawnPoliceCar(p_building);
            Money -= economyManager.prices[3];
            UIManager.Instance.UpdateMoneyLabel();
            ReturnToNormalState();
        }
    }
    private bool HaveEnoughMoney(int p_itemIndex)
    {
        if (p_itemIndex < 0 || p_itemIndex >= economyManager.prices.Count)
            return false;
        if (Money >= economyManager.prices[p_itemIndex])
            return true;
        return false;
    }
    private void ReturnToNormalState()
    {
        GameState = GameStateType.NORMAL;
        UIManager.Instance.UpdateCancelButton(0);
        UIManager.Instance.EnableNormalUI(true);
    }

    public void SellButtonClicked()
    {
        if (GameState == GameStateType.EDITING_POLICE_CAMERA)
        {
            Money += EconomyManager.Instance.GetUnitSellPrice(0);
            patrolManager.RemoveCamera(selectedCamera);
        }
        else if (GameState == GameStateType.EDITING_POLICE_MAN)
        {
            Money += EconomyManager.Instance.GetUnitSellPrice(1);
            patrolManager.RemovePoliceUnit(selectedUnit);
        }
        else if (GameState == GameStateType.EDITING_POLICE_STATION)
        {
            Money += EconomyManager.Instance.GetUnitSellPrice(2);
            specialBuildingsManager.RemovePoliceStation(selectedStation);
        }
        else if (GameState == GameStateType.EDITING_POLICE_CAR)
        {
            Money += EconomyManager.Instance.GetUnitSellPrice(3);
            patrolManager.RemovePoliceUnit(selectedUnit);
        }
        UIManager.Instance.UpdateMoneyLabel();
        ReturnToNormalState();
    }
    public void RotateButtonClicked()
    {
        selectedCamera.RotateCamera();
    }
    public void CancelButtonClicked()
    {
        ReturnToNormalState();
    }
    public void ChangeToEditingStateState(GameStateType p_state)
    {
        GameState = p_state;
        if ((int)GameState >= 6 && (int)GameState <= 10)
        {
            UIManager.Instance.EnableNormalUI(false);
            UIManager.Instance.UpdateSellButtonLabel(((int)GameState) - 6);
            if (GameState == GameStateType.EDITING_POLICE_CAMERA)
                UIManager.Instance.rotateButton.gameObject.SetActive(true);
            else
                UIManager.Instance.rotateButton.gameObject.SetActive(false);
        }
        else
        {
            UIManager.Instance.EnableNormalUI(true);
        }
    }

    public void ShowEndGameScreen(bool p_isWin)
    {
        Time.timeScale = 0f;
        GameState = GameStateType.END_GAME;
        endScreenContanier.SetActive(true);
        winContanier.SetActive(p_isWin);
        lossContanier.SetActive(!p_isWin);
    }
    public void PlayAgainButtonClicked()
    {
        GameState = GameStateType.NORMAL;
        Instance = null;
        SceneManager.LoadScene("GameScene");
    }
    public void MenuButtonClicked()
    {
        GameState = GameStateType.NORMAL;
        Instance = null;
        SceneManager.LoadScene("TitleScreen");
    }
}
